{***************************************************************
 *
 * Unit Name: EllipsisLabel
 * Purpose  : An extension to a normal label. This has an extra
 *            property for doing automatic ellipsis of text or
 *            directory paths.
 * Author   : Sren Berg Glasius (e-mail: Soeren@Glasius.DK)
 * Status   : Freeware, but please send e-mail if you find it
 *            useful.
 * History  : 15-04-98: First release - and probably final :-)
 *
 ****************************************************************}

unit EllipsisLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TEllipsis = (elNone, elEnd, elPath);

  TEllipsisLabel = class(TCustomLabel)
  private
    FEllipsis: TEllipsis;
    procedure SetEllipsis(Value: TEllipsis);
    procedure DoDrawText(var Rect: TRect; Flags: Word);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Ellipsis: TEllipsis
      read FEllipsis
      write SetEllipsis
      default elNone;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

// Constructor
constructor TEllipsisLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEllipsis := elNone;
end;

// Control the Ellipsis property
procedure TEllipsisLabel.SetEllipsis(Value: TEllipsis);
begin
  if FEllipsis <> Value then begin
    FEllipsis := Value;
    if Value <> elNone then
      AutoSize := False;
    Invalidate;
  end;
end;

// A rewrite of Borlands DoDrawText.
procedure TEllipsisLabel.DoDrawText(var Rect: TRect; Flags: Word);
var
  Text: string;
begin
  Text := Caption;
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or ShowAccelChar and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawTextEx(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags,Nil);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawTextEx(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags,nil);
  end
  else
    DrawTextEx(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags,nil);
end;

// A rewrite of Borlands paint.
procedure TEllipsisLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
  Ellipsis: array[TEllipsis] of Word = (0, DT_END_ELLIPSIS, DT_PATH_ELLIPSIS);
var
  Rect, CalcRect: TRect;
  DrawStyle: Integer;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    DrawStyle := DT_EXPANDTABS or WordWraps[WordWrap] or Alignments[Alignment] or
      Ellipsis[FEllipsis];
    { Calculate vertical layout }
    if Layout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if Layout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    DoDrawText(Rect, DrawStyle);
  end;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TEllipsisLabel]);
end;

end.

